////////////////////////////////////////////////////////////////////
// Programme de fichiers binaires pour Chrono Cross
////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////
// Nemesis pour Terminus Traduction (tt_nemesis@yahoo.fr)
// Date de cration :		10/09/03
// Dernire modification :	10/09/03
////////////////////////////////////////////////////////////////////

// Entete
#include "Binaire.h"

/*------------------------------------------------------------------
  Cration d'un handle sur un fichier binaire
--------------------------------------------------------------------
 <- fich : fichier standard ouvert
 <- params : paramtres de la fonction fopen
 -> fichier binaire
------------------------------------------------------------------*/
Bin_TFich *Bopen (char *fich, char *params)
{
	// Fichier binaire
	Bin_TFich *bin_f=(Bin_TFich *)malloc(sizeof(Bin_TFich));

	// Paramtres valables ?
	if (fich!=NULL && params!=NULL)
		bin_f->fich=fopen(fich,params);
	else
		bin_f->fich=NULL;
	bin_f->fin=OK;
	bin_f->bit=0;
	bin_f->octet=0;

	return(bin_f);
}

/*------------------------------------------------------------------
  Finit l'octet en cours en ajoutant des 0
--------------------------------------------------------------------
 <- fich : fichier standard ouvert
------------------------------------------------------------------*/
void Bacheve (Bin_TFich *bin_f)
{
	// Cas de la fin d'un octet
	if (bin_f->bit>7) {
		fputc(bin_f->octet,bin_f->fich);
		bin_f->octet=0;
		bin_f->bit=0;
	} else {
		// Ecriture du morceau d'octet manquant
		Bwrite_M((_int32)0,8-bin_f->bit,bin_f);
	}
}

/*------------------------------------------------------------------
  Renvoie la position dans le fichier argument
--------------------------------------------------------------------
 <- fich : fichier standard ouvert
 -> position depuis le dbut du fichier
------------------------------------------------------------------*/
_int32 Btell (Bin_TFich *bin_f)
{
	return ftell(bin_f->fich);
}

/*------------------------------------------------------------------
  Lecture d'un entier en Intel
--------------------------------------------------------------------
 <- nb : nombre d'lments  lire
 <- bin_f : fichier standard ouvert
 -> entier lu
------------------------------------------------------------------*/
_int32 Bread_I (_int8 nb, Bin_TFich *bin_f)
{
	_int32 val;
	_int8 c;

	// Initialisation
	val=0;
	c=0;
	// Paramtres valables ?
	if (bin_f->fich!=NULL && nb>0 && nb<=32) {
		while (nb>=8 && bin_f->fin==OK) {
			val+=Bread_M(8,bin_f)<<(8*c);
			c++;
			nb-=8;
		}
		val+=Bread_M(nb,bin_f)<<(8*c);
	}

	return(val);
}

/*------------------------------------------------------------------
  Lecture d'un entier en Motorola
--------------------------------------------------------------------
 <- nb : nombre d'lments  lire
 <- bin_f : fichier standard ouvert
 -> entier lu
------------------------------------------------------------------*/
_int32 Bread_M (_int8 nb, Bin_TFich *bin_f)
{
	_int32 val;
	_int8 lus=0;

	// Initialisation
	val=0;
	// Paramtres valables ?
	if (bin_f->fich!=NULL && nb>0 && nb<=32) {
		for (lus=0;lus<nb && bin_f->fin==OK;lus++) {
			// Mise  jour du "tampon"
			bin_f->bit--;
			if (bin_f->bit<0) {
				bin_f->octet=fgetc(bin_f->fich);
				// Cas de la fin du fichier
				if (feof(bin_f->fich)!=0)
					bin_f->fin=ER;
				bin_f->bit=val%256;
				bin_f->bit=7;
			}
			// Obtention d'un bit
			val=val*2+((bin_f->octet>>bin_f->bit)%2);
		}
	}

	return(val);
}

/*------------------------------------------------------------------
  Ecriture d'un entier en Motorola
--------------------------------------------------------------------
 <- val : lment  crire
 <- nb : nombre de bits  crire
 <- bin_f : fichier standard ouvert
------------------------------------------------------------------*/
void Bwrite_M (_int32 val, _int8 nb, Bin_TFich *bin_f)
{
	_int8 ecr=0;

	// Paramtres valables ?
	if (bin_f->fich!=NULL && nb>0 && nb<=32) {
		for (ecr=nb-1;ecr>=0;ecr--) {
			if (bin_f->bit>7) {
				fputc(bin_f->octet,bin_f->fich);
				bin_f->octet=0;
				bin_f->bit=0;
			}
			// Ecriture d'un bit
			bin_f->octet=bin_f->octet*2+((val>>ecr)%2);
			// Mise  jour du "tampon"
			bin_f->bit++;
		}
	}
}

/*------------------------------------------------------------------
  Ecriture d'un entier en Intel
--------------------------------------------------------------------
 <- val : lment  crire
 <- nb : nombre de bits  crire
 <- bin_f : fichier standard ouvert
------------------------------------------------------------------*/
void Bwrite_I (_int32 val, _int8 nb, Bin_TFich *bin_f)
{
	_int8 c;

	// Initialisation
	c=0;
	// Paramtres valables ?
	if (bin_f->fich!=NULL && nb>0 && nb<=32) {
		while (nb>=8 && bin_f->fin==OK) {
			Bwrite_M((val>>(8*c)%256),8,bin_f);
			c++;
			nb-=8;
		}
		Bwrite_M(val%256,nb,bin_f);
	}
}

/*------------------------------------------------------------------
  Fermeture d'un fichier binaire
--------------------------------------------------------------------
 <- bin_f : fichier standard  fermer
 <- type : mthode d'ouverture IN ou OUT
------------------------------------------------------------------*/
void Bclose (Bin_TFich *bin_f, int type)
{
	if (bin_f->fich!=NULL) {
		if (type==IN)
			Bacheve(bin_f);
		fclose(bin_f->fich);
		bin_f->fich=NULL;
		bin_f->bit=0;
		bin_f->octet=0;
		bin_f->fin=OK;
	}
}
