////////////////////////////////////////////////////////////////////
// Gestion de l'Iso de Chrono Cross
////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////
// Nemesis pour Terminus Traduction (tt_nemesis@yahoo.fr)
// Date de cration :		30/07/03
// Dernire modification :	14/07/04
////////////////////////////////////////////////////////////////////

// Entete
#include "Iso.h"

// Fichier iso
FILE *iso_f;
// Nom de l'iso (chemin)
char iso_nom[ISO_NOM_MAX];
// Mode d'ouverture
int iso_mode=IN;

// Entete de secteurs sur l'iso
unsigned _int8 SECT_DEBUT[] =		{0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00};
unsigned _int8 SECT_SUB_FICH[] =	{0x00,0x00,0x08,0x00,0x00,0x00,0x08,0x00};
unsigned _int8 SECT_SUB_FIN[] =		{0x00,0x00,0x89,0x00,0x00,0x00,0x89,0x00};

/*------------------------------------------------------------------
  Mode d'accs  l'iso
--------------------------------------------------------------------
 <- mode : L(ecture) ou E(criture) pour Avant/Apres
------------------------------------------------------------------*/
void Iso_Mode (char *mode)
{
	if (mode[0]==LECTURE) {
		iso_mode=IN;
		Main_Message_S("Mode lecture d'image.\n","",OK);
	} else {
		iso_mode=OUT;
		Main_Message_S("Mode ecriture d'image.\n","",OK);
	}
}

/*------------------------------------------------------------------
  Initialisation de l'iso (ouverture)
------------------------------------------------------------------*/
void Iso_Ouvre (void)
{
	Main_Message_S("Ouverture de l'image %s... ",ISO_IN,OK);

	if (iso_mode==IN) {
		sprintf(iso_nom,"%s%s.%s",DIR_ISO_IN,ISO_IN,EXT_ISO);
		iso_f=fopen(iso_nom,"rb");
	} else {
		sprintf(iso_nom,"%s%s.%s",DIR_ISO_OUT,ISO_OUT,EXT_ISO);
		iso_f=fopen(iso_nom,"r+b");
	}

	// Erreur d'ouverture ?
	if (iso_f==NULL)
		Main_Message_S("Erreur d'ouverture.\n","",ER);
	Main_OK();
}

/*------------------------------------------------------------------
  Fermeture de l'iso
------------------------------------------------------------------*/
void Iso_Ferme (void)
{
	Main_Message_S("Fermeture de l'image... ","",OK);
	// Erreur de fermeture ;)
	if (iso_f==NULL)
		Main_Message_S("Iso non ouverte.\n","",ER);
	else
		fclose(iso_f);
	Main_OK();
}

// Toutes les fonctions qui suivent ne testent
// pas si l'iso a bien t ouverte, attention !
/*------------------------------------------------------------------
  Renvoie de la position dans l'iso
------------------------------------------------------------------*/
_int64 Iso_Position (void)
{
	return(ftell(iso_f));
}

/*------------------------------------------------------------------
  Saut sur un secteur quelconque de l'iso
--------------------------------------------------------------------
 <- secteur : secteur o l'on souhaite se rendre
------------------------------------------------------------------*/
void Iso_Secteur_Saut (unsigned _int32 secteur)
{
	fseek(iso_f,secteur*SECTEUR_TOTAL,SEEK_SET);
}

/*------------------------------------------------------------------
  Saut par dessus les ECC
------------------------------------------------------------------*/
void Iso_ECC_Saut (void)
{
	fseek(iso_f,SECTEUR_ECC,SEEK_CUR);
}

/*------------------------------------------------------------------
  Saut par dessus l'entete de secteur
------------------------------------------------------------------*/
void Iso_Entete_Saut (void)
{
	fseek(iso_f,SECTEUR_ENTETE,SEEK_CUR);
}

/*------------------------------------------------------------------
  Modification de l'entte d'un secteur
--------------------------------------------------------------------
 <- type : type de secteur SECT_TFICH, SECT_TFIN, SECT_TAUTRE
------------------------------------------------------------------*/
void Iso_Entete_Modif (int type)
{
	unsigned _int8 debut[SECTEUR_DEBUT];

	// Lecture du dbut de secteur (toujours pareil)
	fread(debut,sizeof(char),SECTEUR_DEBUT,iso_f);
	// Est-ce bien une entete de secteur (debug)
	// Saut du numro de secteur
	fseek(iso_f,SECTEUR_NUM,SEEK_CUR);
	// Type de secteur (SUB)
	switch (type) {
	case SECT_TFICH:
		fwrite(SECT_SUB_FICH,sizeof(char),SECTEUR_SUB,iso_f);
		break;
	case SECT_TFIN:
		fwrite(SECT_SUB_FIN,sizeof(char),SECTEUR_SUB,iso_f);
		break;
	default:
		fseek(iso_f,SECTEUR_SUB,SEEK_CUR);
	}
}
/*------------------------------------------------------------------
  Test de dbut de secteur (SECT_DEBUT)
--------------------------------------------------------------------
 <- debut : balise de secteur  tester
 -> renvoie 0 si identiques, premier octet de diffrence
	en partant de la fin sinon
------------------------------------------------------------------*/
int Iso_Test_Debut (unsigned _int8 debut[SECTEUR_DEBUT])
{
	int c;

	for(c=SECTEUR_DEBUT-1;c>=0 && debut[c]==SECT_DEBUT[c];c--);
	if (c==-1)
		c=OUI;
	else
		c=NON;

	return(c);
}

/*------------------------------------------------------------------
  Test de sous-entete (SECT_SUB)
--------------------------------------------------------------------
 <- debut : balise de secteur  tester
 -> renvoie 0 si identiques, premier octet de diffrence
	en partant de la fin sinon
------------------------------------------------------------------*/
int Iso_Test_Sub (unsigned _int8 sub[SECTEUR_SUB])
{
	int c;

	for(c=SECTEUR_SUB-1;c>=0 && sub[c]==SECT_SUB_FIN[c];c--);
	if (c>=0) {
		for(c=SECTEUR_SUB-1;c>=0 && sub[c]==SECT_SUB_FICH[c];c--);
		if (c>=0)
			c=SECT_TAUTRE;
		else
			c=SECT_TFICH;
	} else
		c=SECT_TFIN;

	return(c);
}

/*------------------------------------------------------------------
  Lecture de l'entte d'un secteur
--------------------------------------------------------------------
 <- test : test a effectuer ou non sur l'entete
 -> renvoie le type de secteur SECT_TFICH, SECT_TFIN, SECT_TAUTRE
------------------------------------------------------------------*/
int Iso_Entete_Lit (int test)
{
	unsigned _int8 debut[SECTEUR_DEBUT];
	unsigned _int8 sub[SECTEUR_SUB];

	// Lecture du dbut de secteur (toujours pareil)
	fread(debut,sizeof(char),SECTEUR_DEBUT,iso_f);
	// Est-ce bien une entete de secteur (debug)
	if (test==OUI) {
		if (Iso_Test_Debut(debut)==NON)
			Main_Message_S("On n'est pas place en"
				" debut de secteur.\n","",ER);
	}
	// Saut du numro de secteur
	fseek(iso_f,SECTEUR_NUM,SEEK_CUR);
	// Type de secteur (SUB)
	fread(sub,sizeof(char),SECTEUR_SUB,iso_f);

	return(Iso_Test_Sub(sub));
}

/*------------------------------------------------------------------
  Saut sur place pour corriger les bugs de fopen(..."r+")
  (cf documentation de la fonction)
------------------------------------------------------------------*/
void Iso_Flush (void)
{
	fseek(iso_f,0L,SEEK_CUR);
}

/*------------------------------------------------------------------
  Lecture d'adresses dans un secteur
------------------------------------------------------------------*/
void Iso_Adresses_Lit (unsigned _int32 *buffer)
{
	fread(buffer,sizeof(_int32),SECTEUR_DONNEES/4,iso_f);
}

/*------------------------------------------------------------------
  Ecriture d'adresses dans un secteur
------------------------------------------------------------------*/
void Iso_Adresses_Ecrit (unsigned _int32 *buffer)
{
	Iso_Secteur_Insere((unsigned char *)buffer);
}

/*------------------------------------------------------------------
  Copie d'un secteur vers un fichier
--------------------------------------------------------------------
 <- fich : fichier o les donnes sont crites
------------------------------------------------------------------*/
void Iso_Secteur_Extrait(FILE *fich)
{
	char buffer[SECTEUR_TOTAL];

	fread(buffer,sizeof(char),SECTEUR_TOTAL,iso_f);	
	fwrite(buffer,sizeof(char),SECTEUR_TOTAL,fich);	
}

/*------------------------------------------------------------------
  Copie des donnees d'un secteur vers un fichier
--------------------------------------------------------------------
 <- fich : fichier o les donnes sont crites
 <- 8*fin : distance d'arret  la fin des donnees
	(dans le cas d'un secteur de fin de fichier)
------------------------------------------------------------------*/
void Iso_Donnees_Extrait(FILE *fich, unsigned _int8 fin)
{
	char buffer[SECTEUR_DONNEES];

	fread(buffer,sizeof(char),SECTEUR_DONNEES,iso_f);	
	fwrite(buffer,sizeof(char),SECTEUR_DONNEES-8*fin,fich);	
}

/*------------------------------------------------------------------
  Copie des donnees d'un buffer vers un secteur (+ EDC/ECC)
--------------------------------------------------------------------
 <- donnees : donnes  insrer
------------------------------------------------------------------*/
void Iso_Secteur_Insere(unsigned char donnees[SECTEUR_DONNEES])
{
	unsigned char buffer[SECTEUR_EDCECC];
	// Position de dbut de secteur
	_int64 pos;

	// Chargement complet du buffer avec l'entte
	pos = Iso_Position()-SECTEUR_ENTETE;
	fseek(iso_f,(long)pos,SEEK_SET);
	fread(buffer,sizeof(char),SECTEUR_ENTETE,iso_f);
	fseek(iso_f,(long)pos,SEEK_SET);
	// Copie des donnes
	memcpy(&buffer[SECTEUR_ENTETE],donnees,SECTEUR_DONNEES);
	// 0 pour les EDC/ECC
	memset(&buffer[SECTEUR_ENTETE+SECTEUR_DONNEES],0,SECTEUR_ECC);

	// Calcul des EDC/ECC
	// (par Yazoo, sa version tant plus rapide que la mienne,
	// je ne pensais pas que l'cart se verrait  ce point entre
	// l'optimise avec table et le calcul bte et mchant)
	do_encode_L2(buffer,MODE_2_FORM_1, 0);

	// Ecriture dans l'iso
	fwrite(buffer,sizeof(char),SECTEUR_TOTAL,iso_f);
	// Test de validit
	if (Iso_Position()%SECTEUR_TOTAL!=0)
		Main_Message_S("On n'est pas aligne"
			" sur un debut de secteur.\n","",ER);
}


/*------------------------------------------------------------------
  Copie des donnees d'un fichier vers un secteur (+ EDC/ECC)
--------------------------------------------------------------------
 <- fich : fichier o les donnes sont lues
------------------------------------------------------------------*/
void Iso_Donnees_Insere(FILE *fich)
{
	unsigned char donnees[SECTEUR_DONNEES];
	// Nombre d'octets lus
	unsigned int nb;

	nb=fread(donnees,sizeof(char),SECTEUR_DONNEES,fich);	
	// Remplissage de la fin de secteur par des 0
	memset(&donnees[nb],0,SECTEUR_DONNEES-nb);

	Iso_Secteur_Insere(donnees);
}
